<?php

  if (isset($_POST['show_add_contact'])) {
    $city_id = $_POST['city_id'];
    show_new_contact_form($city_id);
  }
  else if (isset($_POST['add_contact'])) {
    $id = add_contact($displayname);
    if ($id !== false) {
      echo "<p>Added contact.</p>\n";
      $parameters = array($displayname, $id);
    }
  }
  else if (isset($_POST['update_contact'])) {
    list($name, $id, $args) = parse_parameters($parameters);
    $q = new ContactQuery;
    $contact = $q->findOneById($id);
    if ($contact) {
      $area = get_contact_area($contact);
      if ($area) $area_id = $area->getId();
      if (update_contact($contact, $area_id) !== false) {
        echo "<p>Updated contact.</p>\n";
        $parameters = array($contact->getDisplayname(), $contact->getId());
      }
    }
    else {
      echo "<p>No such contact!</p>\n";
    }
  }
  else if ($_POST['search_contact']) {
    header(sprintf("Location: http%s://%s/%s/search/%s", ($_SERVER['HTTPS']) ? "s" : "", $_SERVER['HTTP_HOST'], $module, urlencode($_POST['search_contact'])));
    exit;
  }
  else if ($_POST['show_in_area']) {
    $q = new AreaQuery;
    $area = $q->findOneById($_POST['area_id']);
    header(sprintf("Location: http%s://%s/%s/in/area/%s/%d", ($_SERVER['HTTPS']) ? "s" : "", $_SERVER['HTTP_HOST'], $module, urlencode($area->getName()), $_POST['area_id']));
    exit;
  }
  else if ($_POST['show_in_city']) {
    $q = new CityQuery;
    $city = $q->findOneById($_POST['city_id']);
    header(sprintf("Location: http%s://%s/%s/in/city/%s/%d", ($_SERVER['HTTPS']) ? "s" : "", $_SERVER['HTTP_HOST'], $module, urlencode($city->getName()), $_POST['city_id']));
    exit;
  }

  function show_contact_summary(&$contact, $editing = false) {
    if ($editing) echo "<p>Contact: <span class=\"strong\">" . htmlspecialchars($contact->getDisplayname()) . "</span>";
    else echo "<br>\nContact " . $contact->getStrongLink();
    $role = $contact->getRole();
    $role_string = get_contact_role_string($contact);
    if ($role_string) echo " $role_string";
    $d = urlencode($contact->getDisplayname());
    $i = $contact->getId();
    if ($role & $GLOBALS['ROLE_DONOR']) echo " " . get_small_link("Donations", "/donation/from/contact/%s/%d", $d, $i);
    if ($role & $GLOBALS['ROLE_REQUESTER']) echo " " . get_small_link("Referred", "/order/from/referrer/%s/%d", $d, $i);
    if ($role & $GLOBALS['ROLE_BENEFICIARY']) {
      echo " " . get_small_link("Orders", "/order/to/beneficiary/%s/%d", $d, $contact->getId());
      if (get_contact_area($contact)) echo " " . get_small_link("Place", "/order/place/for/beneficiary/%s/%d", $d, $i);
    }
    if (check_admin(1)) {
      echo " " . $contact->getDeleteLink();
    }
    $area = get_contact_area($contact);
    echo " in " . $area->getLink();
    $city = get_contact_city($contact);
    echo ", " . $city->getLink(get_city_displayname($city));
  }

  function show_contacts($offset, $per_page, $address_ids) {
    $q = new ContactQuery;
    $contacts = $q->filterByAddressId($address_ids)->orderByForename()->orderBySurname()->find();
    if (count($contacts)) {
      foreach ($contacts as $contact) show_contact_summary($contact);
    }
    else echo " none";
  }

  function search_contacts($offset, $per_page, $search) {
    $q = new ContactQuery;
    $contacts = $q->filterByDisplayname("%$search%")->find();
    echo "<p>Contacts matching '" . htmlspecialchars($search) . "':";
    if (count($contacts)) {
      foreach ($contacts as $contact) show_contact_summary($contact);
    }
    else echo "none";
    echo "</p>\n";

    $address_ids = array();
    $q = new AddressQuery;
    $addresses = $q->filterByLine("%$search%")->find();
    foreach ($addresses as $address) $address_ids[] = $address->getId();
    $q = new AddressQuery;
    $addresses = $q->filterByPostcode("%$search%")->find();
    foreach ($addresses as $address) $address_ids[] = $address->getId();

    $q = new ContactQuery;
    $contacts = $q->filterByAddressId($address_ids)->find();
    echo "<p>Contacts in address '" . htmlspecialchars($search) . "':";
    if (count($contacts)) {
      foreach ($contacts as $contact) show_contact_summary($contact);
    }
    else echo "none";
    echo "</p>\n";
  }

  function show_city_contacts($offset, $per_page, $city_name, $city_id = null) {
    if (isset($city_id)) $city = get_city_by_id($city_id);
    else if ($city_name) $city = get_city_by_name($city_name);
    if ($city) {
      $q = new AreaQuery;
      $areas = $q->filterByCityId($city->getId())->find();
      $area_ids = array();
      foreach ($areas as $area) $area_ids[] = $area->getId();

      $q = new AddressQuery;
      $addresses = $q->filterByAreaId($area_ids)->find();
      $address_ids = array();
      foreach ($addresses as $address) $address_ids[] = $address->getId();

      echo "<p>Contacts in city " . $city->getLink(get_city_displayname($city)) . ":";
      return show_contacts($offset, $per_page, $address_ids);
    }
    else echo "<p>No such city!</p>\n";
  }

  function show_area_contacts($offset, $per_page, $area_name, $area_id = null) {
    if (isset($area_id)) $area = get_area_by_id($area_id);
    else if ($area_name) $area = get_area_by_name($area_name);
    if ($area) {
      $q = new AddressQuery;
      $addresses = $q->filterByAreaId($area->getId())->find();
      $address_ids = array();
      foreach ($addresses as $address) $address_ids[] = $address->getId();

      echo "<p>Contacts in area " . $area->getLink() . ":";
      return show_contacts($offset, $per_page, $address_ids);
    }
    else echo "<p>No such area!</p>\n";
  }

  function show_contact_areas_form($city_id = null) {
    $areas = get_city_areas($city_id);
    if (! count($areas)) {
      echo "<p>No <a href=\"/area\">areas</a>!</p>\n";
      return;
    }

    $candidates = array();
    foreach ($areas as $area) {
      if (! count(get_area_contacts($area->getId()))) continue;
      $candidates[] = $area;
    }
    if (! count($candidates)) return;

    echo "<p>Show contacts in area\n";
    echo "<select name=\"area_id\">\n";
    foreach ($candidates as $area) {
      option("area_id", $area->getId(), get_area_displayname($area));
    }
    echo "</select>\n";
    submit("show_in_area", "Show");
  }

  function show_contact_cities_form($city_id = null) {
    $q = new CityQuery;
    $cities = $q->orderByName()->find();

    if (! count($cities)) {
      echo "<p>No <a href=\"/city\">cities</a>!</p>\n";
      return;
    }

    $candidates = array();
    foreach ($cities as $city) {
      if (! count(get_city_contacts($city->getId()))) continue;
      $candidates[] = $city;
    }
    if (! count($candidates)) return;

    echo "<p>Show contacts in city\n";
    echo "<select name=\"city_id\">\n";
    foreach ($candidates as $city) {
      option("city_id", $city->getId(), get_city_displayname($city), $city_id);
    }
    echo "</select>\n";
    submit("show_in_city", "Show");
  }

  function show_contact_search_form() {
    echo "<p>Search for contacts:";
    input("search_contact");
    echo "<input type=\"submit\" value=\"Search\">\n";
  }

  function show_contact_forms($city_id) {
    form("noprint standout");
    show_contact_areas_form($city_id);
    show_contact_cities_form($city_id);
    show_contact_search_form();
    end_form();
  }

  function show_contact_role_form($role) {
    return show_role_form($role, $GLOBALS['contact_roles']);
  }

  function show_contact_form($contact = null, $new = false) {
    global $contact_roles, $parcel_sizes, $parcel_contents;

    if (! $contact) $contact = new Contact;
    else if ($contact->getRole() & ($GLOBALS['ROLE_BENEFICIARY'] | $GLOBALS['ROLE_REQUESTER'])) {
      $state_mask = $GLOBALS['STATE_ANY'];
      $state_mask &= ~$GLOBALS['STATE_DELIVERED'];
      $state_mask &= ~$GLOBALS['STATE_CANCELLED'];

      $orders = get_contact_orders($contact, $state_mask);

      if (count($orders)) {
        echo "<tr>\n";
        echo "  <td colspan=2><strong>Outstanding orders:</strong></td>\n";
        echo "</tr>\n";

        echo "<tr>\n";
        echo "  <td colspan=2 class=\"history\">\n";
        foreach ($orders as $order) {
          echo "    " . get_order_summary($order) . "<br>\n";
        }
        echo "  </td>\n";
        echo "</tr>\n";
      }
    }

    /* Role. */
    echo "<tr>\n";
    echo "  <td>Role</td>\n";
    echo "  <td>"; show_contact_role_form($contact->getRole()); echo "</td>\n";
    echo "</tr>\n";

    /* Date added. */
    if (! $new) {
      echo "<tr>\n";
      echo "  <td>Registered</td>\n";
      echo "  <td>" . $contact->getAdded() . "</td>\n";
      echo "</tr>\n";
    }

    /* Forename. */
    echo "<tr>\n";
    echo "  <td>Forename</td>\n";
    echo "  <td>"; input("forename", $contact->getForename()); echo "</td>\n";
    echo "</tr>\n";

    /* Middle names. */
    echo "<tr>\n";
    echo "  <td>Middle name(s)</td>\n";
    echo "  <td>"; input("middle", $contact->getMiddle()); echo "</td>\n";
    echo "</tr>\n";

    /* Surname. */
    echo "<tr>\n";
    echo "  <td>Surname</td>\n";
    echo "  <td>"; input("surname", $contact->getSurname()); echo "</td>\n";
    echo "</tr>\n";

    /* Display name. */
    echo "<tr>\n";
    echo "  <td>Display name (if not concatenation of above)</td>\n";
    echo "  <td>"; input("displayname", $contact->getDisplayname()); echo "</td>\n";
    echo "</tr>\n";

    /* Address. */
    $address = get_contact_address($contact);
    if (! $address) $address = new Address;
    echo "<tr>\n";
    echo "  <td>Address</td>\n";
    echo "  <td>"; textarea("address", $address->getLine()); echo "</td>\n";
    echo "</tr>\n";

    /* Postcode. */
    echo "<tr>\n";
    echo "  <td>Postcode</td>\n";
    echo "  <td>"; input("postcode", $address->getPostcode()); echo get_address_map_link($address); echo "</td>\n";
    echo "</tr>\n";

    /* Telephone. */
    echo "<tr>\n";
    echo "  <td>Telephone</td>\n";
    echo "  <td>"; input("telephone1", $contact->getTelephone1()); echo "</td>\n";
    echo "</tr>\n";
    echo "<tr>\n";
    echo "  <td>Alternative telephone</td>\n";
    echo "  <td>"; input("telephone2", $contact->getTelephone2()); echo "</td>\n";
    echo "</tr>\n";

    /* Email. */
    echo "<tr>\n";
    echo "  <td>Email</td>\n";
    echo "  <td>"; input("email", $contact->getEmail()); echo "</td>\n";
    echo "</tr>\n";

    /* Area. */
    $area = get_contact_area($contact);
    if ($area) $area_id = $area->getId();
    echo "<tr>\n";
    echo "  <td>Area</td>\n";
    echo "  <td><select name=\"area_id\">\n";
    $areas = get_city_areas();
    foreach ($areas as $area) {
      option("area_id", $area->getId(), get_area_displayname($area), $area_id);
    }
    echo "  </select></td>\n";
    echo "</tr>\n";

    /* Parcel type. */
    echo "<tr>\n";
    echo "  <td>Family unit</td>\n";
    echo "  <td><select name=\"parcel_size\">\n";
    $mask = 1 << count($parcel_sizes);
    for ($i = 0; $i < count($parcel_sizes); $i++) {
      option("parcel_size", 1 << $i, $parcel_sizes[$i], $contact->getParcel() % $mask);
    }
    echo "</select></td>\n";
    echo "</tr>\n";

    /* Parcel contents. */
    echo "<tr>\n";
    echo "  <td>Dietary requirements</td>\n";
    echo "  <td>";
    for ($i = count($parcel_sizes); $i < count($parcel_contents); $i++) {
      if (1 << $i == $GLOBALS['PARCEL_TOILETRY']) continue;
      echo "  <input type=\"checkbox\" name=\"parcel_$i\"";
      if ($contact->getParcel() & (1 << $i)) echo " checked";
      echo ">$parcel_contents[$i]\n";
    }
    echo "</td>\n";
    echo "</tr>\n";

    /* Notes. */
    echo "<tr>\n";
    echo "  <td>Notes</td>\n";
    echo "  <td>"; textarea("notes", $contact->getNotes()); echo "</td>\n";
    echo "</tr>\n";
  }

  function show_new_contact_form($city_id = null) {
    if (! check_admin(1)) return;

    $areas = get_city_areas($city_id);
    if (! count($areas)) {
      echo "<p>No <a href=\"/area\">areas</a>!</p>\n";
      return;
    }

    form("noprint");
    echo "<p>Add a new contact:</p>\n";

    echo "<table>\n";
    show_contact_form($contact, true);

    echo "<tr>\n";
    echo "  <td colspan=2>"; submit("add_contact", "Add"); echo "</td></tr>\n";
    echo "</tr>\n";
    echo "</table>\n";
    end_form();
  }

  function show_add_new_contact_form() {
    if (! check_admin(1)) return;

    $q = new CityQuery;
    $cities = $q->find();
    if (! count($cities)) {
      echo "<p>No <a href=\"/city\">cities</a>!</p>\n";
      return;
    }

    form("noprint standout");
    echo "<p>Add a new contact in <select name=\"city_id\">\n";
    foreach ($cities as $city) {
      option("city_id", $city->getId(), get_city_displayname($city));
    }
    echo "</select>";
    submit("show_add_contact", "Proceed");
    echo "</p>\n";
    end_form();
  }

  function update_contact(&$contact, $area_id, $new = false) {
    global $contact_roles, $parcel_sizes, $parcel_contents;

    $role = 0;
    for ($i = 0; $i < count($contact_roles); $i++) {
      if ($_POST['role_' . $i] == "on") $role |= (1 << $i);
    }

    /* Staff can place orders. */
    if ($role & (1 << 0)) $role |= (1 << 2);

    $forename = $_POST['forename'];
    $middle = $_POST['middle'];
    $surname = $_POST['surname'];
    $displayname = $_POST['displayname'];

    if (! $forename && ! $surname) {
      echo "<p>Must have either a forename or surname!</p>\n";
      return false;
    }
    if ($middle && ! ($forename && $surname)) {
      echo "<p>Must have both a forename or surname for middle name(s) to make sense!</p>\n";
      return false;
    }

    if (! $displayname) {
      $displayname = $forename;
      if ($middle) $displayname .= " $middle";
      if ($forename) $displayname .= " ";
      $displayname .= $surname;
      echo "<p>Display name will be $displayname.</p>\n";
    }

    /* Get address. */
    $area_id = $_POST['area_id'];
    $line = $_POST['address'];
    $postcode = $_POST['postcode'];
    $q = new AddressQuery;
    /* XXX: Finding by area properly? */
    $address = $q->filterByAreaId($area_id)->filterByLine($line)->filterByPostcode($postcode)->findOneOrCreate();
    if ($address->isNew()) {
      /* Changing address. */
      //if (! $new)
      /*
        XXX: Check for other contacts at the old address.
        Make this a new address if there are others, but
        provide a link to update other contacts.
      */
      try {
        $address->save();
      }
      catch (Exception $e) {
        echo "<p>Error adding $line.</p>\n";
        return false;
      }
    }

    $telephone1 = $_POST['telephone1'];
    $telephone2 = $_POST['telephone2'];
    $email = $_POST['email'];
    $parcel = $_POST['parcel_size'];
    for ($i = count($parcel_sizes); $i < count($parcel_contents); $i++) {
      if ($_POST['parcel_' . $i] == "on") $parcel |= (1 << $i);
    }
    $notes = $_POST['notes'];

    $contact->setRole($role);
    $contact->setForename($forename);
    $contact->setMiddle($middle);
    $contact->setSurname($surname);
    $contact->setDisplayname($displayname);
    $contact->setTelephone1($telephone1);
    $contact->setTelephone2($telephone2);
    $contact->setEmail($email);
    $contact->setParcel($parcel);
    $contact->setNotes($notes);
    $contact->setAddressId($address->getId());

    try {
      $contact->save();
    }
    catch (Exception $e) {
      if ($new) echo "<p>Error adding $displayname.</p>\n";
      else echo "<p>Error updating $displayname.</p>\n";
      return false;
    }

    return true;
  }

  function add_contact(&$name) {
    if (! check_admin(1, "add a contact")) return;

    $area_id = $_POST['area_id'];
    if (! is_numeric($area_id)) {
      echo "<p>Invalid area!</p>\n";
      return false;
    }

    $area = get_area_by_id($area_id);
    if (! $area) {
      echo "<p>No such area!</p>\n";
      return false;
    }

    $contact = new Contact;
    if (! update_contact($contact, $area_id, true)) return false;
    return $contact->getId();
  }

  function confirm_delete_contact($name, $id = null, &$city_id = null) {
    if (! check_admin(1, "delete a contact")) return;

    if (isset($id)) $contact = get_contact_by_id($id);
    else $contact = get_contact_by_name($name);
    if (! $contact) return false;

    echo "<h3>Confirm deletion</h3>\n";
    echo "<p>You must confirm deletion of contact " . $contact->getDisplayname() . ": " . $contact->getDeleteLink(true) . "</p>\n";
  }

  function delete_contact($name, $id = null, &$city_id = null) {
    if (! check_admin(1, "delete a contact")) return;

    if (isset($id)) $contact = get_contact_by_id($id);
    else $contact = get_contact_by_name($name);
    if (! $contact) return false;

    ///* Remember city ID for dropdown. */
    //$city_id = $area->getCityId();

    try {
      $contact->delete();
      echo "<p>Deleted contact.</p>\n";
    }
    catch (Exception $e) {
      echo "<p>Error deleting $name!</p>\n";
      return false;
    }

    return true;
  }

  function show_contact($name, &$id = null) {
    if (isset($id)) $contact = get_contact_by_id($id);
    else $contact = get_contact_by_name($name);
    if (! $contact) return;

    form();
    show_contact_summary($contact, true);
    echo ": ";
    echo "\n</p>";

    echo "<table>\n";
    show_contact_form($contact);

    if (check_admin(1)) {
      echo "<tr>\n";
      echo "  <td colspan=2>";
      submit("update_contact", "Update");
      echo "</td>\n";
      echo "</tr>\n";
    }

    echo "</table>\n";
    end_form();
  }

  /* /contact/in/area/Cambridge/1 */
  if (count($parameters)) {
    if ($parameters[0] == "in") {
      switch ($parameters[1]) {
        case "area":
          $area_id = $parameters[3];
          $_POST['area_id'] = $area_id;
          $q = new AreaQuery;
          $area = $q->findOneById($area_id);
          $city = get_area_city($area);
          if ($city) $city_id = $city->getId();
          show_area_contacts(0, 10, $parameters[2], $area_id);
        break;

        case "city":
          $city_id = $parameters[3];
          $_POST['city_id'] = $city_id;
          $q = new CityQuery;
          $city = $q->findOneById($city_id);
          show_city_contacts(0, 10, $parameters[2], $city_id);
        break;
      }
    }
    else if ($parameters[0] == "search") {
      search_contacts(0, 10, urldecode($parameters[1]));
    }
  }
  list($name, $id, $args) = parse_parameters($parameters);
  //echo "<p>$name($id) " . print_r($args, true) . "</p>\n";
  if (count($args)) {
    switch ($args[0]) {
      case "delete":
        confirm_delete_contact($name, $id);
      break;

      case "confirmdelete":
        delete_contact($name, $id);
      break;
    }
  }
  else if (isset($name)) show_contact($name, $id);

  show_contact_forms($city_id);
  show_add_new_contact_form($city_id);

?>
