<?php

  if (isset($_POST['show_reports'])) {
    header(sprintf("Location: http%s://%s/%s/from/%s/to/%s", ($_SERVER['HTTPS']) ? "s" : "", $_SERVER['HTTP_HOST'], $module, $_POST['from'], $_POST['to']));
    exit;
  }

  function show_reports_form($from = null, $to = null) {
    form("noprint standout");
    echo "<p>Show reports covering the period from ";

    /* Default to last month. */
    list($y, $m, $d) = explode('-', date('Y-m-d', time()));
    $latest = "$y-$m-$d";
    $now = mktime(0, 0, 0, $m, $d, $y);
    $first = mktime(0, 0, 0, $m, 1, $y);
    $last = $first - 86400;
    $date = date('Y-m-d', $last);
    if (is_null($to)) $to = $date;
    list($y, $m, $d) = explode('-', $date);
    $first = mktime(0, 0, 0, $m, 1, $y);
    if (is_null($from)) $from = date('Y-m-d', $first);
    $date = $first;
    for ($i = 0; $i < 2; $i++) {
      $date -= 86400;
      list ($y, $m, $d) = explode('-', date('Y-m-d', $date));
      $date = mktime(0, 0, 0, $m, 1, $y);
    }
    $oldest = date('Y-m-d', $date);
    $then = $date;

    echo "<select name=\"from\">\n";
    for ($date = $then; $date <= $now; $date += 86400) {
      option("from", date('Y-m-d', $date), date('j F Y', $date), $from);
    }
    echo "</select> to <select name=\"to\">\n";
    for ($date = $then; $date <= $now; $date += 86400) {
      option("to", date('Y-m-d', $date), date('j F Y', $date), $to);
    }
    echo "</select>\n";

    submit("show_reports", "Show");
    end_form();
  }

  function check_report_dates($from, $to) {
    list($y, $m, $d) = explode('-', $from);
    if (! checkdate($m, $d, $y)) {
      echo "<p>Invalid report start date!</p>\n";
      return false;
    }
    $start = mktime(0, 0, 0, $m, $d, $y);

    list($y, $m, $d) = explode('-', $to);
    if (! checkdate($m, $d, $y)) {
      echo "<p>Invalid report end date!</p>\n";
      return false;
    }
    $end = mktime(0, 0, 0, $m, $d, $y);

    if ($end < $start) {
      echo "<p>Report end date is earlier than start date!</p>\n";
      return false;
    }

    return true;
  }

  function show_order_report(&$order_state_ids) {
    echo "<h3>Orders by week</h3>\n";

    $q = new OrderStateQuery;
    $q->filterById($order_state_ids);
    $q->withColumn('yearweek(updated)', 'week');
    $q->withColumn('count(*)', 'count');
    $q->addGroupByColumn('week')->orderByUpdated();
    $rows = $q->find();
    $week_offset = 0;
    $week = 1;
    $last_week = 0;
    $total = 0;
    echo "<table>\n";
    foreach ($rows as $row) {
      /* Convert week of year to date range. */
      if (! $week_offset) $week_offset = $row->getWeek() - 1;
      else $week = $row->getWeek() - $week_offset;
      $total += $row->getCount();
      /* Fill in missing weeks. XXX */
      for ($missing_week = $last_week + 1; $missing_week < $week; $missing_week++) {
        echo "<tr>\n";
        printf("  <td>0</td><td>Week %d</td>\n", $missing_week);
        echo "</tr>\n";
      }
      echo "<tr>\n";
      printf("  <td>%d</td><td>Week %d</td>\n", $row->getCount(), $week);
      echo "</tr>\n";
      $last_week = $week;
    }
    echo "<tr>\n";
    echo "  <td class=\"strong\">$total</td><td class=\"strong\">Total</td>\n";
    echo "</tr>\n";
    echo "</table>\n";
  }

  function show_requester_report(&$order_ids) {
    echo "<h3>Orders by referrer</h3>\n";

    $q = new OrderQuery;
    $q->filterById($order_ids);
    $q->withColumn('count(*)', 'count');
    $q->groupByRequesterId()->addDescendingOrderByColumn('count');
    $rows = $q->find();
    $total = 0;
    echo "<table>\n";
    /* XXX Join! */
    foreach ($rows as $row) {
      echo "<tr>\n";
      $requester = get_contact_by_id($row->getRequesterId());
      printf("  <td>%d</td><td>%s</td>\n", $row->getCount(), htmlspecialchars($requester->getDisplayname()));
      $total += $row->getCount();
      echo "</tr>\n";
    }
    echo "<tr>\n";
    echo "  <td class=\"strong\">$total</td><td class=\"strong\">Total</td>\n";
    echo "</tr>\n";
    echo "</table>\n";
  }

  function show_reports($from, $to) {
    if (! check_report_dates($from, $to)) return;

    echo "<p>Showing reports for the period <strong>$from</strong> to <strong>$to</strong>.</p>\n";

    /* Get orders. */
    $order_ids = array();
    $order_state_ids = array();
    /* XXX: Order 51 changed to state delivered in May then updated in June. */
    $dbh = Propel::getConnection();
    $sth = $dbh->prepare("select * from OrderState o where updated=(select min(updated) from OrderState where order_id=o.order_id and state & " . $GLOBALS['STATE_DELIVERED'] . ") and updated between '$from' and '$to'");
    $sth->execute();
    $order_states = OrderStatePeer::populateObjects($sth);
    foreach ($order_states as $order_state) {
      $order_ids[] = $order_state->getOrderId();
      $order_state_ids[] = $order_state->getId();
    }
    $q = new OrderQuery;
    $q->filterById($order_ids);

    if (! count($order_ids)) {
      echo "<p>No results!</p>\n";
      return;
    }

    show_order_report($order_state_ids);
    show_requester_report($order_ids);
  }

  if (count($parameters)) {
    if ($parameters[0] == "from") {
      $from = $parameters[1];
      if ($parameters[2] == "to") $to = $parameters[3];
      show_reports($from, $to);
    }
  }
  show_reports_form($from, $to);
?>
